/*****************************************************************************
 Header file containing global camera structures 
*****************************************************************************/

/******************************************************************************
                              Analog Speed Codes
******************************************************************************/

#define A40KHZ    0         /* Indicates 40KHz analog card */
#define A50KHZ    1         /* Indicates 50KHz analog card */
#define A200KHZ   2         /* Indicates 200KHz analog card */
#define A500KHZ   3         /* Indicates 500KHz analog card */

/* Structure definitions. */

/* Each temperature reading is an array of five values:
 * [0] entry is the last read value, [1] is the min, [2] max, 
 * [3] is the mean value, [4] is the number of readings taken in this sample.
 */

#define TEMP_CUR	0
#define TEMP_MIN	1
#define TEMP_MAX	2
#define TEMP_MEAN	3
#define TEMP_NO		4

#define MAX_SAMPLES	277 /* Max. no. of pixels available for temp.display */
     /* File header (fixed size 512 bytes) for file. */ 
 

typedef struct {
	int tei[5];	/* Enclosure internal ambient temperature. */
	int tee[5];	/* Enclosure external ambient temperature. */
	int td[5];	/* Dome temperature. */
	int tcc[5];	/* CCD temperature, coarse (Not used) */
	int tcf[5];	/* CCD temperature, fine */
	int tfw[5];	/* Filter wheel temperature. */
} temp_t;

typedef struct {
	unsigned char camera;	/* Camera number */
	unsigned char fw;	/* Filter wheel position */
	unsigned char afb_id;	/* AFB ID */
	unsigned char portno;	/* COMMx port number */
	temp_t   temp;		/* Temperatures. */
	unsigned heater;	/* Heater status */
	unsigned fan_stat;	/* Fan status */
	unsigned acpower;	/* AC power status. */
	unsigned plus5;		/* DC voltages. */
	unsigned plus15;
	unsigned minus15;
	unsigned preamp;	/* Preamp on/off during exposure. */
	unsigned exp_scale;	/* Exposure scale 1=ms, 1000 = sec, 60000=min */
	unsigned sequence;	/* Unique file sequence number. */
        int      ems_handle;	/* EMS handle for image from this camera. */
        long	image_size;	/* Image size in bytes. */
	int	version;	/* file/program version. ((major*16) + minor) */
	struct  dosdate_t dt;	/* date of exposure */
	struct  dostime_t tm;	/* time of exposure */
	char	exposure_type;	/* Exposure type */
	int	pixel_max;	/* Max pixel intensity for last image */
	int	pixel_min;	/* Min pixel intensity for last image */
	int	cam_opmode;	/* Camera operating mode ( 1 or 2 cameras ) */
	char	lens_desc[6];	/* Description of camera lens. */
	char	filter_desc[6];	/* Description of current filter. */
	char	location[81];	/* place name, geodetic co-ordinates, etc. */
	char	azimuth[11];	/* Camera orientation */
	char	operator[4];	/* Operators initials */
	char	jpeg_qual[4];	/* JPEG quality factor (100 = no compression) */
	char	comp_fact[4];	/* Compression factor (1.0 = no compression) */
	char	os_version[4];	/* Operating system version. */
} cam_misc;


/******************************************************************************
                     AT200 High-Level Control Structures
******************************************************************************/

typedef struct {
  ccd_descript ccd;       /* The current CCD characterization */
  unsigned at200_id:5;    /* The ID of the current AT200/M200 [0..31]
                             The I/O address of the AT200/M200 is calculated
                             from this value as: 0x200 + (at200_id * 16) */
  unsigned frame:1;       /* TRUE=Using frame transfer mode */
  unsigned open_delay:5;  /* Milliseconds to delay when opening the shutter */
  unsigned close_delay:5; /* Milliseconds to delay when closing the shutter */
  unsigned preamp_delay:7;/* Milliseconds to delay after turning the preamp on */
  unsigned speed:3;       /* Analog Speed code (see below) */
  unsigned data_bits:5;   /* Number of data bits in the A/D converter */
} cam_sys;

typedef struct {
  unsigned far * far * ram; /* Pointer to array of pointers to data segments.
                               Each entry in this array of pointers points to
                               an area of memory that will be used to hold 2K
                               pixels (4K Bytes) from the controller.
                               Calculate the number of required entries as
                               ((pixels - 1) / 2048) + 1 */
  unsigned long exposure;   /* Milliseconds to expose CCD */
  unsigned gain;            /* Gain: Zero=>1x, Non-Zero=>4x */
  unsigned mpp_mode;        /* Zero=Don't use MPP, Non-Zero=Use MPP */
  unsigned corg;            /* Columns to skip before readout */
  unsigned rorg;            /* Rows to skip before readout */
  unsigned cols;            /* Columns to readout */
  unsigned rows;            /* Rows to readout */
  unsigned sbin;            /* Column (serial) binning factor */
  unsigned pbin;            /* Row (parallel) binning factor */
  unsigned clr_cnt;         /* Cycles to clear before exposure */
  unsigned row_shift;       /* Rows to shift in the shift() function */
} cam_exp;

/* In a main program, symbols defined by EXTRNL are defined,
 * In a sub-function, they are referenced via 'extern'.
 */

/******************************************************************************
            KI interpreter start-up data.
******************************************************************************/

/* File sequence numbers for specified data types. */

typedef struct {
	struct		dosdate_t dt;	/* effective date for sequence #s */
	unsigned int	auroral[2][5];
	unsigned int	airglow[2][5];
	unsigned int	bias[2];
	unsigned int	calib[2][5];
	unsigned int	dark[2];
	unsigned int	flat[2][5];
	char            last_cam0_img_flnm[80]; /* Last image for camera 0 */
	char            last_cam1_img_flnm[80]; /* Last image for camera 1 */
} KI_DATA;

/******************************************************************************
                     AFB data conversion constants.
******************************************************************************/

typedef struct {
	unsigned int id;
	int	tei[2];		/* high, low constants */
	int	tee[2];
	int	td[2];
	int	tcc[2];
	int	tcf[2];
	int	tfw[2];
	int	plus5;
	int	plus15;
	int	minus15;
} AFB_CONST;

/*****************************************************************************
		Structure for array of temperature display data.
*****************************************************************************/

typedef struct {
	int	tei;
	int	tee;
	int	td;
	int	tcf;
	int	tfw;
	struct dostime_t tm;
} t_record;

typedef union file_hdr_tag
     {
       struct cam_structs_tag 
       { 	
         cam_misc misc;
         cam_sys  sys;
         cam_exp  exp;
       } structs;
       char     filler[512];
     } file_hdr;

#ifdef MAIN_PROG
#define EXTRNL
#else
#define EXTRNL extern
#endif

EXTRNL int  	cmdsrc, cmdlnx;		/* Global index to startup command line. */
EXTRNL char 	cmdln[CMDLNSIZE]; 	/* Startup Command line */
EXTRNL FILE 	*f1;			/* Startup Command file */

EXTRNL int	abrt;	    /* Global abort flag. */
EXTRNL int	fdata;	    /* File handle for file sequence data */

EXTRNL KI_DATA  ki_data;    /* File sequence numbers. */
EXTRNL cam_sys std0_sys;    /* standard cam_sys and cam_exp used as defaults */
EXTRNL cam_sys std1_sys;

EXTRNL cam_exp std0_exp;
EXTRNL cam_exp std1_exp;

EXTRNL cam_misc std0_misc;
EXTRNL cam_misc std1_misc;

/* Filter descriptions for 2 cameras, 5 filters each, 6 bytes per desc. */

EXTRNL char	filter_desc[2][5][6];	/* Filter descriptions for cameras */
EXTRNL AFB_CONST afb_const[2];		/* AFB data conversion constants. */

EXTRNL int      sel_cam;		/* Selected camera number - 0 or 1 */
EXTRNL cam_sys  *sel_sys;		/* Pointer to camera structures. */
EXTRNL cam_exp  *sel_exp;
EXTRNL cam_misc *sel_misc;

EXTRNL unsigned far *ems_data;		/* Pointer to EMM page frame. */

EXTRNL char	omsg[255];		/* Output msg to stdout [and log] */
EXTRNL char	imagepath[PATHSIZE];	/* Pathname for camera images. */
EXTRNL char	filespath[PATHSIZE];	/* Pathname for start_up files, etc. */
EXTRNL char	scriptpath[PATHSIZE];	/* Pathname for XEQ script files. */

/* Globally visible date and time. */

EXTRNL struct  dosdate_t dt;
EXTRNL struct  dostime_t tm;

/* Array for storing histogram data. */

EXTRNL unsigned long histogram[ 2 ][ 256 ];

/* Pointer to bitmap, initialized in genbitmap() */

EXTRNL unsigned char * bitmap;

/* Two arrays of structures to hold data for graphical temperature display */
	  
EXTRNL t_record t_history0[MAX_SAMPLES];	 
EXTRNL t_record t_history1[MAX_SAMPLES];

EXTRNL int sampleptr, no_samples, exposure_type;
EXTRNL file_hdr header0;
EXTRNL file_hdr header1;
EXTRNL file_hdr *sel_hdr;
EXTRNL int exec_compress, do_compress[2];
